@@ -44,15 +44,15 @@
  * using wildcards, e.g. "standard::*" will return all of the keys in the
  * "standard" namespace.
  *
- * Values are stored within the list in #GFileAttributeValue structures.
- * Values can store different types, listed in the enum #GFileAttributeType.
- * Upon creation of a #GFileAttributeValue, the type will be set to
- * %G_FILE_ATTRIBUTE_TYPE_INVALID.
- *
  * The list of possible attributes for a filesystem (pointed to by a #GFile) is
  * available as a #GFileAttributeInfoList. This list is queryable by key names
  * as indicated earlier.
  *
+ * Information is stored within the list in #GFileAttributeInfo structures.
+ * The info structure can store different types, listed in the enum
+ * #GFileAttributeType. Upon creation of a #GFileAttributeInfo, the type will
+ * be set to %G_FILE_ATTRIBUTE_TYPE_INVALID.
+ *
  * Classes that implement #GFileIface will create a #GFileAttributeInfoList and
  * install default keys and values for their given file system, architecture,
  * and other possible implementation details (e.g., on a UNIX system, a file
@@ -963,6 +963,7 @@ g_file_attribute_info_list_unref (GFileAttributeInfoList *list)
       for (i = 0; i < list->n_infos; i++)
         g_free (list->infos[i].name);
       g_array_free (priv->array, TRUE);
+      g_free (list);
     }
 }
 
