@@ -2,12 +2,10 @@
  *
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  */
@@ -25,8 +25,8 @@
 #include <glib.h>
 #include <gfileiostream.h>
 #include <gseekable.h>
+#include "gsimpleasyncresult.h"
 #include "gasyncresult.h"
-#include "gtask.h"
 #include "gcancellable.h"
 #include "gioerror.h"
 #include "gfileoutputstream.h"
@@ -34,28 +34,31 @@
 
 
 /**
- * GFileIOStream:
+ * SECTION:gfileiostream
+ * @short_description:  File read and write streaming operations
+ * @include: gio/gio.h
+ * @see_also: #GIOStream, #GFileInputStream, #GFileOutputStream, #GSeekable
  *
- * `GFileIOStream` provides I/O streams that both read and write to the same
+ * GFileIOStream provides io streams that both read and write to the same
  * file handle.
  *
- * `GFileIOStream` implements [iface@Gio.Seekable], which allows the I/O
+ * GFileIOStream implements #GSeekable, which allows the io
  * stream to jump to arbitrary positions in the file and to truncate
  * the file, provided the filesystem of the file supports these
  * operations.
  *
- * To find the position of a file I/O stream, use [method@Gio.Seekable.tell].
+ * To find the position of a file io stream, use
+ * g_seekable_tell().
  *
- * To find out if a file I/O stream supports seeking, use
- * [method@Gio.Seekable.can_seek]. To position a file I/O stream, use
- * [method@Gio.Seekable.seek]. To find out if a file I/O stream supports
- * truncating, use [method@Gio.Seekable.can_truncate]. To truncate a file I/O
- * stream, use [method@Gio.Seekable.truncate].
- *
- * The default implementation of all the `GFileIOStream` operations
- * and the implementation of [iface@Gio.Seekable] just call into the same
- * operations on the output stream.
+ * To find out if a file io stream supports seeking, use g_seekable_can_seek().
+ * To position a file io stream, use g_seekable_seek().
+ * To find out if a file io stream supports truncating, use
+ * g_seekable_can_truncate(). To truncate a file io
+ * stream, use g_seekable_truncate().
  *
+ * The default implementation of all the #GFileIOStream operations
+ * and the implementation of #GSeekable just call into the same operations
+ * on the output stream.
  * Since: 2.22
  **/
 
@@ -82,15 +85,14 @@ static GFileInfo *g_file_io_stream_real_query_info_finish (GFileIOStream    *str
 							   GAsyncResult         *result,
 							   GError              **error);
 
+G_DEFINE_TYPE_WITH_CODE (GFileIOStream, g_file_io_stream, G_TYPE_IO_STREAM,
+			 G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE,
+						g_file_io_stream_seekable_iface_init));
+
 struct _GFileIOStreamPrivate {
   GAsyncReadyCallback outstanding_callback;
 };
 
-G_DEFINE_TYPE_WITH_CODE (GFileIOStream, g_file_io_stream, G_TYPE_IO_STREAM,
-                         G_ADD_PRIVATE (GFileIOStream)
-			 G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE,
-						g_file_io_stream_seekable_iface_init))
-
 static void
 g_file_io_stream_seekable_iface_init (GSeekableIface *iface)
 {
@@ -104,14 +106,16 @@ g_file_io_stream_seekable_iface_init (GSeekableIface *iface)
 static void
 g_file_io_stream_init (GFileIOStream *stream)
 {
-  stream->priv = g_file_io_stream_get_instance_private (stream);
+  stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (stream,
+					      G_TYPE_FILE_IO_STREAM,
+					      GFileIOStreamPrivate);
 }
 
 /**
  * g_file_io_stream_query_info:
  * @stream: a #GFileIOStream.
  * @attributes: a file attribute query string.
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
  * @error: a #GError, %NULL to ignore.
  *
  * Queries a file io stream for the given @attributes.
@@ -163,7 +167,7 @@ g_file_io_stream_query_info (GFileIOStream      *stream,
     info = class->query_info (stream, attributes, cancellable, error);
   else
     g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                         _("Stream doesn’t support query_info"));
+                         _("Stream doesn't support query_info"));
 
   if (cancellable)
     g_cancellable_pop_current (cancellable);
@@ -190,12 +194,11 @@ async_ready_callback_wrapper (GObject *source_object,
  * g_file_io_stream_query_info_async:
  * @stream: a #GFileIOStream.
  * @attributes: a file attribute query string.
- * @io_priority: the [I/O priority](iface.AsyncResult.html#io-priority) of the
- *   request
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (scope async): a #GAsyncReadyCallback
- *   to call when the request is satisfied
- * @user_data: the data to pass to callback function
+ * @io_priority: the <link linkend="gio-GIOScheduler">I/O priority</link>
+ *     of the request.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (scope async): callback to call when the request is satisfied
+ * @user_data: (closure): the data to pass to callback function
  *
  * Asynchronously queries the @stream for a #GFileInfo. When completed,
  * @callback will be called with a #GAsyncResult which can be used to
@@ -224,9 +227,10 @@ g_file_io_stream_query_info_async (GFileIOStream     *stream,
 
   if (!g_io_stream_set_pending (io_stream, &error))
     {
-      g_task_report_error (stream, callback, user_data,
-                           g_file_io_stream_query_info_async,
-                           error);
+      g_simple_async_report_take_gerror_in_idle (G_OBJECT (stream),
+					    callback,
+					    user_data,
+					    error);
       return;
     }
 
@@ -235,7 +239,7 @@ g_file_io_stream_query_info_async (GFileIOStream     *stream,
   stream->priv->outstanding_callback = callback;
   g_object_ref (stream);
   klass->query_info_async (stream, attributes, io_priority, cancellable,
-                           async_ready_callback_wrapper, user_data);
+			      async_ready_callback_wrapper, user_data);
 }
 
 /**
@@ -263,8 +267,6 @@ g_file_io_stream_query_info_finish (GFileIOStream     *stream,
 
   if (g_async_result_legacy_propagate_error (result, error))
     return NULL;
-  else if (g_async_result_is_tagged (result, g_file_io_stream_query_info_async))
-    return g_task_propagate_pointer (G_TASK (result), error);
 
   class = G_FILE_IO_STREAM_GET_CLASS (stream);
   return class->query_info_finish (stream, result, error);
@@ -278,7 +280,7 @@ g_file_io_stream_query_info_finish (GFileIOStream     *stream,
  * This must be called after the stream has been written
  * and closed, as the etag can change while writing.
  *
- * Returns: (nullable) (transfer full): the entity tag for the stream.
+ * Returns: the entity tag for the stream.
  *
  * Since: 2.22
  **/
@@ -651,6 +653,8 @@ g_file_io_stream_real_query_info_finish (GFileIOStream     *stream,
 static void
 g_file_io_stream_class_init (GFileIOStreamClass *klass)
 {
+  g_type_class_add_private (klass, sizeof (GFileIOStreamPrivate));
+
   klass->tell = g_file_io_stream_real_tell;
   klass->can_seek = g_file_io_stream_real_can_seek;
   klass->seek = g_file_io_stream_real_seek;
