@@ -70,6 +70,8 @@ spawn (const gchar *expected_stdout,
   g_ptr_array_add (array, NULL);
   args = (gchar **) g_ptr_array_free (array, FALSE);
 
+  va_end (ap);
+
   data = g_slice_new (ChildData);
   data->expected_stdout = expected_stdout;
 
@@ -449,8 +451,6 @@ test_actions (void)
 int
 main (int argc, char **argv)
 {
-  g_type_init ();
-
   g_test_init (&argc, &argv, NULL);
 
   g_test_dbus_unset ();
