@@ -19,8 +19,10 @@
  */
 
 #include <gio/gio.h>
+#include <glib/gstdio.h>
 
 #ifdef G_OS_UNIX
+#include <fcntl.h>
 #include <gio/gunixinputstream.h>
 #include <gio/gunixoutputstream.h>
 #endif
@@ -44,7 +46,7 @@ poll_source_callback (GPollableInputStream *in,
   g_assert_cmpstr (buf, ==, "x");
 
   *success = TRUE;
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 static gboolean
@@ -55,7 +57,7 @@ check_source_readability_callback (gpointer user_data)
 
   readable = g_pollable_input_stream_is_readable (in);
   g_assert_cmpint (readable, ==, expected);
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 static gboolean
@@ -71,7 +73,7 @@ write_callback (gpointer user_data)
 
   check_source_readability_callback (GINT_TO_POINTER (TRUE));
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 static gboolean
@@ -79,7 +81,7 @@ check_source_and_quit_callback (gpointer user_data)
 {
   check_source_readability_callback (user_data);
   g_main_loop_quit (loop);
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -92,6 +94,9 @@ test_streams (void)
   GSource *poll_source;
   gboolean success = FALSE;
 
+  g_assert (g_pollable_input_stream_can_poll (in));
+  g_assert (g_pollable_output_stream_can_poll (G_POLLABLE_OUTPUT_STREAM (out)));
+
   readable = g_pollable_input_stream_is_readable (in);
   g_assert (!readable);
 
@@ -136,7 +141,7 @@ test_streams (void)
 static void
 test_pollable_unix (void)
 {
-  int pipefds[2], status;
+  int pipefds[2], status, fd;
 
   status = pipe (pipefds);
   g_assert_cmpint (status, ==, 0);
@@ -148,7 +153,48 @@ test_pollable_unix (void)
 
   g_object_unref (in);
   g_object_unref (out);
+
+  /* Non-pipe/socket unix streams are not pollable */
+  fd = g_open ("/dev/null", O_RDWR, 0);
+  g_assert_cmpint (fd, !=, -1);
+  in = G_POLLABLE_INPUT_STREAM (g_unix_input_stream_new (fd, FALSE));
+  out = g_unix_output_stream_new (fd, FALSE);
+
+  g_assert (!g_pollable_input_stream_can_poll (in));
+  g_assert (!g_pollable_output_stream_can_poll (G_POLLABLE_OUTPUT_STREAM (out)));
+
+  g_object_unref (in);
+  g_object_unref (out);
+  close (fd);
 }
+
+static void
+test_pollable_converter (void)
+{
+  GConverter *converter;
+  GError *error = NULL;
+  GInputStream *ibase;
+  int pipefds[2], status;
+
+  status = pipe (pipefds);
+  g_assert_cmpint (status, ==, 0);
+
+  ibase = G_INPUT_STREAM (g_unix_input_stream_new (pipefds[0], TRUE));
+  converter = G_CONVERTER (g_charset_converter_new ("UTF-8", "UTF-8", &error));
+  g_assert_no_error (error);
+
+  in = G_POLLABLE_INPUT_STREAM (g_converter_input_stream_new (ibase, converter));
+  g_object_unref (converter);
+  g_object_unref (ibase);
+
+  out = g_unix_output_stream_new (pipefds[1], TRUE);
+
+  test_streams ();
+
+  g_object_unref (in);
+  g_object_unref (out);
+}
+
 #endif
 
 static void
@@ -228,11 +274,11 @@ int
 main (int   argc,
       char *argv[])
 {
-  g_type_init ();
   g_test_init (&argc, &argv, NULL);
 
 #ifdef G_OS_UNIX
   g_test_add_func ("/pollable/unix", test_pollable_unix);
+  g_test_add_func ("/pollable/converter", test_pollable_converter);
 #endif
   g_test_add_func ("/pollable/socket", test_pollable_socket);
 
