@@ -2,27 +2,27 @@
  *
  *  Copyright 2001,2005 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * The Gnome Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * The Gnome Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the Gnome Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *   Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 
 #include <locale.h>
 #include <string.h>
-#ifdef HAVE_WCHAR_H
+#ifdef __STDC_ISO_10646__
 #include <wchar.h>
 #endif
 
@@ -37,12 +37,11 @@
 #include "gstrfuncs.h"
 #include "gtestutils.h"
 #include "gcharset.h"
+#ifndef __STDC_ISO_10646__
 #include "gconvert.h"
-
-#if SIZEOF_WCHAR_T == 4 && defined(__STDC_ISO_10646__)
-#define GUNICHAR_EQUALS_WCHAR_T 1
 #endif
 
+
 #ifdef _MSC_VER
 /* Workaround for bug in MSVCR80.DLL */
 static gsize
@@ -67,18 +66,14 @@ msc_strxfrm_wrapper (char       *string1,
  * @str2: a UTF-8 encoded string
  * 
  * Compares two strings for ordering using the linguistically
- * correct rules for the [current locale](running.html#locale).
+ * correct rules for the <link linkend="setlocale">current locale</link>. 
  * When sorting a large number of strings, it will be significantly 
  * faster to obtain collation keys with g_utf8_collate_key() and 
  * compare the keys with strcmp() when sorting instead of sorting 
  * the original strings.
  * 
- * If the two strings are not comparable due to being in different collation
- * sequences, the result is undefined. This can happen if the strings are in
- * different language scripts, for example.
- *
- * Returns: < 0 if @str1 compares before @str2, 
- *   0 if they compare equal, > 0 if @str1 compares after @str2.
+ * Return value: &lt; 0 if @str1 compares before @str2, 
+ *   0 if they compare equal, &gt; 0 if @str1 compares after @str2.
  **/
 gint
 g_utf8_collate (const gchar *str1,
@@ -108,7 +103,7 @@ g_utf8_collate (const gchar *str1,
   g_free (str2_utf16);
   g_free (str1_utf16);
 
-#elif defined(HAVE_WCHAR_H) && defined(GUNICHAR_EQUALS_WCHAR_T)
+#elif defined(__STDC_ISO_10646__)
 
   gunichar *str1_norm;
   gunichar *str2_norm;
@@ -124,7 +119,7 @@ g_utf8_collate (const gchar *str1,
   g_free (str1_norm);
   g_free (str2_norm);
 
-#else
+#else /* !__STDC_ISO_10646__ */
 
   const gchar *charset;
   gchar *str1_norm;
@@ -161,12 +156,12 @@ g_utf8_collate (const gchar *str1,
   g_free (str1_norm);
   g_free (str2_norm);
 
-#endif
+#endif /* __STDC_ISO_10646__ */
 
   return result;
 }
 
-#if defined(HAVE_WCHAR_H) && defined(GUNICHAR_EQUALS_WCHAR_T)
+#if defined(__STDC_ISO_10646__) || defined(HAVE_CARBON)
 /* We need UTF-8 encoding of numbers to encode the weights if
  * we are using wcsxfrm. However, we aren't encoding Unicode
  * characters, so we can't simply use g_unichar_to_utf8.
@@ -213,7 +208,7 @@ utf8_encode (char *buf, wchar_t val)
 
   return retval;
 }
-#endif
+#endif /* __STDC_ISO_10646__ || HAVE_CARBON */
 
 #ifdef HAVE_CARBON
 
@@ -223,31 +218,28 @@ collate_key_to_string (UCCollationValue *key,
 {
   gchar *result;
   gsize result_len;
-  long *lkey = (long *) key;
+  gsize i;
 
-  /* UCCollationValue format:
-   *
-   * UCCollateOptions (32/64 bits)
-   * SizeInBytes      (32/64 bits)
-   * Value            (8 bits arrey)
-   *
-   * UCCollateOptions: ordering option mask of the collator
-   * used to create the key. Size changes on 32-bit / 64-bit
-   * hosts. On 64-bits also the extra half-word seems to have
-   * some extra (unknown) meaning.
-   * SizeInBytes: size of the whole structure, in bytes
-   * (including UCCollateOptions and SizeInBytes fields). Size
-   * changes on 32-bit & 64-bit hosts.
-   * Value: array of bytes containing the comparison weights.
-   * Seems to have several sub-strings separated by \001 and \002
-   * chars. Also, experience shows this is directly strcmp-able.
-   */
+  /* Pretty smart algorithm here: ignore first eight bytes of the
+   * collation key. It doesn't produce results equivalent to
+   * UCCompareCollationKeys's, but the difference seems to be only
+   * that UCCompareCollationKeys in some cases produces 0 where our
+   * comparison gets -1 or 1. */
+
+  if (key_len * sizeof (UCCollationValue) <= 8)
+    return g_strdup ("");
+
+  result_len = 0;
+  for (i = 8; i < key_len * sizeof (UCCollationValue); i++)
+    /* there may be nul bytes, encode byteval+1 */
+    result_len += utf8_encode (NULL, *((guchar*)key + i) + 1);
 
-  result_len = lkey[1];
   result = g_malloc (result_len + 1);
-  memcpy (result, &lkey[2], result_len);
-  result[result_len] = '\0';
+  result_len = 0;
+  for (i = 8; i < key_len * sizeof (UCCollationValue); i++)
+    result_len += utf8_encode (result + result_len, *((guchar*)key + i) + 1);
 
+  result[result_len] = 0;
   return result;
 }
 
@@ -372,16 +364,12 @@ carbon_collate_key_for_filename (const gchar *str,
  * The results of comparing the collation keys of two strings 
  * with strcmp() will always be the same as comparing the two 
  * original keys with g_utf8_collate().
- *
- * Note that this function depends on the [current locale](running.html#locale).
- *
- * Note that the returned string is not guaranteed to be in any
- * encoding, especially UTF-8. The returned value is meant to be
- * used only for comparisons.
  * 
- * Returns: (transfer full) (type filename): a newly allocated string.
- *   The contents of the string are only meant to be used when sorting.
- *   This string should be freed with g_free() when you are done with it.
+ * Note that this function depends on the 
+ * <link linkend="setlocale">current locale</link>.
+ * 
+ * Return value: a newly allocated string. This string should
+ *   be freed with g_free() when you are done with it.
  **/
 gchar *
 g_utf8_collate_key (const gchar *str,
@@ -394,7 +382,7 @@ g_utf8_collate_key (const gchar *str,
   g_return_val_if_fail (str != NULL, NULL);
   result = carbon_collate_key (str, len);
 
-#elif defined(HAVE_WCHAR_H) && defined(GUNICHAR_EQUALS_WCHAR_T)
+#elif defined(__STDC_ISO_10646__)
 
   gsize xfrm_len;
   gunichar *str_norm;
@@ -406,18 +394,16 @@ g_utf8_collate_key (const gchar *str,
 
   str_norm = _g_utf8_normalize_wc (str, len, G_NORMALIZE_ALL_COMPOSE);
 
-  g_return_val_if_fail (str_norm != NULL, NULL);
-
   xfrm_len = wcsxfrm (NULL, (wchar_t *)str_norm, 0);
   result_wc = g_new (wchar_t, xfrm_len + 1);
   wcsxfrm (result_wc, (wchar_t *)str_norm, xfrm_len + 1);
 
-  for (i = 0; i < xfrm_len; i++)
+  for (i=0; i < xfrm_len; i++)
     result_len += utf8_encode (NULL, result_wc[i]);
 
   result = g_malloc (result_len + 1);
   result_len = 0;
-  for (i = 0; i < xfrm_len; i++)
+  for (i=0; i < xfrm_len; i++)
     result_len += utf8_encode (result + result_len, result_wc[i]);
 
   result[result_len] = '\0';
@@ -426,9 +412,9 @@ g_utf8_collate_key (const gchar *str,
   g_free (str_norm);
 
   return result;
-#else
+#else /* !__STDC_ISO_10646__ */
 
-  gsize xfrm_len = 0;
+  gsize xfrm_len;
   const gchar *charset;
   gchar *str_norm;
 
@@ -441,7 +427,7 @@ g_utf8_collate_key (const gchar *str,
   if (g_get_charset (&charset))
     {
       xfrm_len = strxfrm (NULL, str_norm, 0);
-      if (xfrm_len < G_MAXINT - 2)
+      if (xfrm_len >= 0 && xfrm_len < G_MAXINT - 2)
         {
           result = g_malloc (xfrm_len + 1);
           strxfrm (result, str_norm, xfrm_len + 1);
@@ -454,7 +440,7 @@ g_utf8_collate_key (const gchar *str,
       if (str_locale)
 	{
 	  xfrm_len = strxfrm (NULL, str_locale, 0);
-	  if (xfrm_len >= G_MAXINT - 2)
+	  if (xfrm_len < 0 || xfrm_len >= G_MAXINT - 2)
 	    {
 	      g_free (str_locale);
 	      str_locale = NULL;
@@ -480,15 +466,15 @@ g_utf8_collate_key (const gchar *str,
     }
 
   g_free (str_norm);
-#endif
+#endif /* __STDC_ISO_10646__ */
 
   return result;
 }
 
 /* This is a collation key that is very very likely to sort before any
- * collation key that libc strxfrm generates. We use this before any
- * special case (dot or number) to make sure that its sorted before
- * anything else.
+   collation key that libc strxfrm generates. We use this before any
+   special case (dot or number) to make sure that its sorted before
+   anything else.
  */
 #define COLLATION_SENTINEL "\1\1\1"
 
@@ -507,19 +493,15 @@ g_utf8_collate_key (const gchar *str,
  * would like to treat numbers intelligently so that "file1" "file10" "file5"
  * is sorted as "file1" "file5" "file10".
  * 
- * Note that this function depends on the [current locale](running.html#locale).
+ * Note that this function depends on the 
+ * <link linkend="setlocale">current locale</link>.
  *
- * Note that the returned string is not guaranteed to be in any
- * encoding, especially UTF-8. The returned value is meant to be
- * used only for comparisons.
- *
- * Returns: (transfer full) (type filename): a newly allocated string.
- *   The contents of the string are only meant to be used when sorting.
- *   This string should be freed with g_free() when you are done with it.
+ * Return value: a newly allocated string. This string should
+ *   be freed with g_free() when you are done with it.
  *
  * Since: 2.8
  */
-gchar *
+gchar*
 g_utf8_collate_key_for_filename (const gchar *str,
 				 gssize       len)
 {
