@@ -80,7 +80,7 @@ test_unichar_character_type (void)
 
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     {
-      g_assert (g_unichar_type (examples[i].c) == examples[i].type);
+      g_assert_cmpint (g_unichar_type (examples[i].c), ==, examples[i].type);
     }
 }
 
@@ -128,12 +128,15 @@ test_unichar_break_type (void)
     { G_UNICODE_BREAK_HANGUL_V_JAMO,       0x1160 },
     { G_UNICODE_BREAK_HANGUL_T_JAMO,       0x11A8 },
     { G_UNICODE_BREAK_HANGUL_LV_SYLLABLE,  0xAC00 },
-    { G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE, 0xAC01 }
+    { G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE, 0xAC01 },
+    { G_UNICODE_BREAK_CONDITIONAL_JAPANESE_STARTER, 0x3041 },
+    { G_UNICODE_BREAK_HEBREW_LETTER,                0x05D0 },
+    { G_UNICODE_BREAK_REGIONAL_INDICATOR,           0x1F1F6 }
   };
 
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     {
-      g_assert (g_unichar_break_type (examples[i].c) == examples[i].type);
+      g_assert_cmpint (g_unichar_break_type (examples[i].c), ==, examples[i].type);
     }
 }
 
@@ -242,11 +245,18 @@ test_unichar_script (void)
     { G_UNICODE_SCRIPT_TAI_VIET,                0xAA80 },
     { G_UNICODE_SCRIPT_BATAK,                   0x1BC0 },
     { G_UNICODE_SCRIPT_BRAHMI,                 0x11000 },
-    { G_UNICODE_SCRIPT_MANDAIC,                 0x0840 }
+    { G_UNICODE_SCRIPT_MANDAIC,                 0x0840 },
+    { G_UNICODE_SCRIPT_CHAKMA,                 0x11100 },
+    { G_UNICODE_SCRIPT_MEROITIC_CURSIVE,       0x109A0 },
+    { G_UNICODE_SCRIPT_MEROITIC_HIEROGLYPHS,   0x10980 },
+    { G_UNICODE_SCRIPT_MIAO,                   0x16F00 },
+    { G_UNICODE_SCRIPT_SHARADA,                0x11180 },
+    { G_UNICODE_SCRIPT_SORA_SOMPENG,           0x110D0 },
+    { G_UNICODE_SCRIPT_TAKRI,                  0x11680 }
   };
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     {
-      g_assert (g_unichar_get_script (examples[i].c) == examples[i].script);
+      g_assert_cmpint (g_unichar_get_script (examples[i].c), ==, examples[i].script);
     }
 }
 
@@ -294,7 +304,7 @@ test_combining_class (void)
   };
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     {
-      g_assert (g_unichar_combining_class (examples[i].c) == examples[i].class);
+      g_assert_cmpint (g_unichar_combining_class (examples[i].c), ==, examples[i].class);
     }
 }
 
@@ -327,20 +337,66 @@ test_mark (void)
   g_assert (!g_unichar_ismark ('a'));
 }
 
+static void
+test_zerowidth (void)
+{
+  g_assert (!g_unichar_iszerowidth (0x00AD));
+  g_assert (!g_unichar_iszerowidth (0x00AD));
+  g_assert (!g_unichar_iszerowidth (0x115F));
+  g_assert (g_unichar_iszerowidth (0x1160));
+  g_assert (g_unichar_iszerowidth (0x11AA));
+  g_assert (g_unichar_iszerowidth (0x11FF));
+  g_assert (!g_unichar_iszerowidth (0x1200));
+  g_assert (g_unichar_iszerowidth (0x200B));
+  g_assert (g_unichar_iszerowidth (0x591));
+}
+
 static void
 test_title (void)
 {
   g_assert (g_unichar_istitle (0x01c5));
   g_assert (g_unichar_istitle (0x1f88));
   g_assert (g_unichar_istitle (0x1fcc));
+  g_assert (!g_unichar_istitle ('a'));
+  g_assert (!g_unichar_istitle ('A'));
 
-  g_assert (g_unichar_totitle (0x01c6) == 0x01c5);
-  g_assert (g_unichar_totitle (0x01c4) == 0x01c5);
-  g_assert (g_unichar_totitle (0x01c5) == 0x01c5);
-  g_assert (g_unichar_totitle (0x1f80) == 0x1f88);
-  g_assert (g_unichar_totitle (0x1f88) == 0x1f88);
-  g_assert (g_unichar_totitle ('a') == 'A');
-  g_assert (g_unichar_totitle ('A') == 'A');
+  g_assert_cmphex (g_unichar_totitle (0x01c6), ==, 0x01c5);
+  g_assert_cmphex (g_unichar_totitle (0x01c4), ==, 0x01c5);
+  g_assert_cmphex (g_unichar_totitle (0x01c5), ==, 0x01c5);
+  g_assert_cmphex (g_unichar_totitle (0x1f80), ==, 0x1f88);
+  g_assert_cmphex (g_unichar_totitle (0x1f88), ==, 0x1f88);
+  g_assert_cmphex (g_unichar_totitle ('a'), ==, 'A');
+  g_assert_cmphex (g_unichar_totitle ('A'), ==, 'A');
+}
+
+static void
+test_cases (void)
+{
+  g_assert_cmphex (g_unichar_toupper ('a'), ==, 'A');
+  g_assert_cmphex (g_unichar_toupper ('A'), ==, 'A');
+  g_assert_cmphex (g_unichar_toupper (0x01C5), ==, 0x01C4);
+  g_assert_cmphex (g_unichar_toupper (0x01C6), ==, 0x01C4);
+  g_assert_cmphex (g_unichar_tolower ('A'), ==, 'a');
+  g_assert_cmphex (g_unichar_tolower ('a'), ==, 'a');
+  g_assert_cmphex (g_unichar_tolower (0x01C4), ==, 0x01C6);
+  g_assert_cmphex (g_unichar_tolower (0x01C5), ==, 0x01C6);
+  g_assert_cmphex (g_unichar_tolower (0x1F8A), ==, 0x1F82);
+  g_assert_cmphex (g_unichar_totitle (0x1F8A), ==, 0x1F8A);
+  g_assert_cmphex (g_unichar_toupper (0x1F8A), ==, 0x1F8A);
+  g_assert_cmphex (g_unichar_tolower (0x1FB2), ==, 0x1FB2);
+  g_assert_cmphex (g_unichar_toupper (0x1FB2), ==, 0x1FB2);
+}
+
+static void
+test_defined (void)
+{
+  g_assert (g_unichar_isdefined (0x0903));
+  g_assert (g_unichar_isdefined (0x20DD));
+  g_assert (g_unichar_isdefined (0x20BA));
+  g_assert (g_unichar_isdefined (0xA806));
+  g_assert (g_unichar_isdefined ('a'));
+  g_assert (!g_unichar_isdefined (0x10C49));
+  g_assert (!g_unichar_isdefined (0x169D));
 }
 
 static void
@@ -420,8 +476,8 @@ test_wide (void)
 
   for (i = 0; i < G_N_ELEMENTS (examples); i++)
     {
-      g_assert (g_unichar_iswide (examples[i].c) == (examples[i].wide == WIDE));
-      g_assert (g_unichar_iswide_cjk (examples[i].c) == (examples[i].wide != NOT_WIDE));
+      g_assert_cmpint (g_unichar_iswide (examples[i].c), ==, (examples[i].wide == WIDE));
+      g_assert_cmpint (g_unichar_iswide_cjk (examples[i].c), ==, (examples[i].wide != NOT_WIDE));
     }
 };
 
@@ -603,7 +659,10 @@ test_decompose_tail (void)
     if (g_unichar_decompose (ch, &a, &b))
       g_assert (!g_unichar_decompose (b, &c, &d));
     else
-      g_assert (a == ch && b == 0);
+      {
+        g_assert_cmpuint (a, ==, ch);
+        g_assert_cmpuint (b, ==, 0);
+      }
 }
 
 static void
@@ -624,13 +683,151 @@ test_fully_decompose_len (void)
 static void
 test_iso15924 (void)
 {
+  const struct {
+    GUnicodeScript script;
+    char four_letter_code[5];
+  } data[] = {
+    { G_UNICODE_SCRIPT_COMMON,             "Zyyy" },
+    { G_UNICODE_SCRIPT_INHERITED,          "Zinh" },
+    { G_UNICODE_SCRIPT_ARABIC,             "Arab" },
+    { G_UNICODE_SCRIPT_ARMENIAN,           "Armn" },
+    { G_UNICODE_SCRIPT_BENGALI,            "Beng" },
+    { G_UNICODE_SCRIPT_BOPOMOFO,           "Bopo" },
+    { G_UNICODE_SCRIPT_CHEROKEE,           "Cher" },
+    { G_UNICODE_SCRIPT_COPTIC,             "Copt" },
+    { G_UNICODE_SCRIPT_CYRILLIC,           "Cyrl" },
+    { G_UNICODE_SCRIPT_DESERET,            "Dsrt" },
+    { G_UNICODE_SCRIPT_DEVANAGARI,         "Deva" },
+    { G_UNICODE_SCRIPT_ETHIOPIC,           "Ethi" },
+    { G_UNICODE_SCRIPT_GEORGIAN,           "Geor" },
+    { G_UNICODE_SCRIPT_GOTHIC,             "Goth" },
+    { G_UNICODE_SCRIPT_GREEK,              "Grek" },
+    { G_UNICODE_SCRIPT_GUJARATI,           "Gujr" },
+    { G_UNICODE_SCRIPT_GURMUKHI,           "Guru" },
+    { G_UNICODE_SCRIPT_HAN,                "Hani" },
+    { G_UNICODE_SCRIPT_HANGUL,             "Hang" },
+    { G_UNICODE_SCRIPT_HEBREW,             "Hebr" },
+    { G_UNICODE_SCRIPT_HIRAGANA,           "Hira" },
+    { G_UNICODE_SCRIPT_KANNADA,            "Knda" },
+    { G_UNICODE_SCRIPT_KATAKANA,           "Kana" },
+    { G_UNICODE_SCRIPT_KHMER,              "Khmr" },
+    { G_UNICODE_SCRIPT_LAO,                "Laoo" },
+    { G_UNICODE_SCRIPT_LATIN,              "Latn" },
+    { G_UNICODE_SCRIPT_MALAYALAM,          "Mlym" },
+    { G_UNICODE_SCRIPT_MONGOLIAN,          "Mong" },
+    { G_UNICODE_SCRIPT_MYANMAR,            "Mymr" },
+    { G_UNICODE_SCRIPT_OGHAM,              "Ogam" },
+    { G_UNICODE_SCRIPT_OLD_ITALIC,         "Ital" },
+    { G_UNICODE_SCRIPT_ORIYA,              "Orya" },
+    { G_UNICODE_SCRIPT_RUNIC,              "Runr" },
+    { G_UNICODE_SCRIPT_SINHALA,            "Sinh" },
+    { G_UNICODE_SCRIPT_SYRIAC,             "Syrc" },
+    { G_UNICODE_SCRIPT_TAMIL,              "Taml" },
+    { G_UNICODE_SCRIPT_TELUGU,             "Telu" },
+    { G_UNICODE_SCRIPT_THAANA,             "Thaa" },
+    { G_UNICODE_SCRIPT_THAI,               "Thai" },
+    { G_UNICODE_SCRIPT_TIBETAN,            "Tibt" },
+    { G_UNICODE_SCRIPT_CANADIAN_ABORIGINAL, "Cans" },
+    { G_UNICODE_SCRIPT_YI,                 "Yiii" },
+    { G_UNICODE_SCRIPT_TAGALOG,            "Tglg" },
+    { G_UNICODE_SCRIPT_HANUNOO,            "Hano" },
+    { G_UNICODE_SCRIPT_BUHID,              "Buhd" },
+    { G_UNICODE_SCRIPT_TAGBANWA,           "Tagb" },
+
+    /* Unicode-4.0 additions */
+    { G_UNICODE_SCRIPT_BRAILLE,            "Brai" },
+    { G_UNICODE_SCRIPT_CYPRIOT,            "Cprt" },
+    { G_UNICODE_SCRIPT_LIMBU,              "Limb" },
+    { G_UNICODE_SCRIPT_OSMANYA,            "Osma" },
+    { G_UNICODE_SCRIPT_SHAVIAN,            "Shaw" },
+    { G_UNICODE_SCRIPT_LINEAR_B,           "Linb" },
+    { G_UNICODE_SCRIPT_TAI_LE,             "Tale" },
+    { G_UNICODE_SCRIPT_UGARITIC,           "Ugar" },
+
+    /* Unicode-4.1 additions */
+    { G_UNICODE_SCRIPT_NEW_TAI_LUE,        "Talu" },
+    { G_UNICODE_SCRIPT_BUGINESE,           "Bugi" },
+    { G_UNICODE_SCRIPT_GLAGOLITIC,         "Glag" },
+    { G_UNICODE_SCRIPT_TIFINAGH,           "Tfng" },
+    { G_UNICODE_SCRIPT_SYLOTI_NAGRI,       "Sylo" },
+    { G_UNICODE_SCRIPT_OLD_PERSIAN,        "Xpeo" },
+    { G_UNICODE_SCRIPT_KHAROSHTHI,         "Khar" },
+
+    /* Unicode-5.0 additions */
+    { G_UNICODE_SCRIPT_UNKNOWN,            "Zzzz" },
+    { G_UNICODE_SCRIPT_BALINESE,           "Bali" },
+    { G_UNICODE_SCRIPT_CUNEIFORM,          "Xsux" },
+    { G_UNICODE_SCRIPT_PHOENICIAN,         "Phnx" },
+    { G_UNICODE_SCRIPT_PHAGS_PA,           "Phag" },
+    { G_UNICODE_SCRIPT_NKO,                "Nkoo" },
+
+    /* Unicode-5.1 additions */
+    { G_UNICODE_SCRIPT_KAYAH_LI,           "Kali" },
+    { G_UNICODE_SCRIPT_LEPCHA,             "Lepc" },
+    { G_UNICODE_SCRIPT_REJANG,             "Rjng" },
+    { G_UNICODE_SCRIPT_SUNDANESE,          "Sund" },
+    { G_UNICODE_SCRIPT_SAURASHTRA,         "Saur" },
+    { G_UNICODE_SCRIPT_CHAM,               "Cham" },
+    { G_UNICODE_SCRIPT_OL_CHIKI,           "Olck" },
+    { G_UNICODE_SCRIPT_VAI,                "Vaii" },
+    { G_UNICODE_SCRIPT_CARIAN,             "Cari" },
+    { G_UNICODE_SCRIPT_LYCIAN,             "Lyci" },
+    { G_UNICODE_SCRIPT_LYDIAN,             "Lydi" },
+
+    /* Unicode-5.2 additions */
+    { G_UNICODE_SCRIPT_AVESTAN,                "Avst" },
+    { G_UNICODE_SCRIPT_BAMUM,                  "Bamu" },
+    { G_UNICODE_SCRIPT_EGYPTIAN_HIEROGLYPHS,   "Egyp" },
+    { G_UNICODE_SCRIPT_IMPERIAL_ARAMAIC,       "Armi" },
+    { G_UNICODE_SCRIPT_INSCRIPTIONAL_PAHLAVI,  "Phli" },
+    { G_UNICODE_SCRIPT_INSCRIPTIONAL_PARTHIAN, "Prti" },
+    { G_UNICODE_SCRIPT_JAVANESE,               "Java" },
+    { G_UNICODE_SCRIPT_KAITHI,                 "Kthi" },
+    { G_UNICODE_SCRIPT_LISU,                   "Lisu" },
+    { G_UNICODE_SCRIPT_MEETEI_MAYEK,           "Mtei" },
+    { G_UNICODE_SCRIPT_OLD_SOUTH_ARABIAN,      "Sarb" },
+    { G_UNICODE_SCRIPT_OLD_TURKIC,             "Orkh" },
+    { G_UNICODE_SCRIPT_SAMARITAN,              "Samr" },
+    { G_UNICODE_SCRIPT_TAI_THAM,               "Lana" },
+    { G_UNICODE_SCRIPT_TAI_VIET,               "Tavt" },
+
+    /* Unicode-6.0 additions */
+    { G_UNICODE_SCRIPT_BATAK,                  "Batk" },
+    { G_UNICODE_SCRIPT_BRAHMI,                 "Brah" },
+    { G_UNICODE_SCRIPT_MANDAIC,                "Mand" },
+
+    /* Unicode-6.1 additions */
+    { G_UNICODE_SCRIPT_CHAKMA,                 "Cakm" },
+    { G_UNICODE_SCRIPT_MEROITIC_CURSIVE,       "Merc" },
+    { G_UNICODE_SCRIPT_MEROITIC_HIEROGLYPHS,   "Mero" },
+    { G_UNICODE_SCRIPT_MIAO,                   "Plrd" },
+    { G_UNICODE_SCRIPT_SHARADA,                "Shrd" },
+    { G_UNICODE_SCRIPT_SORA_SOMPENG,           "Sora" },
+    { G_UNICODE_SCRIPT_TAKRI,                  "Takr" },
+  };
+  guint i;
+
   g_assert_cmphex (0, ==, g_unicode_script_to_iso15924 (G_UNICODE_SCRIPT_INVALID_CODE));
   g_assert_cmphex (0x5A7A7A7A, ==, g_unicode_script_to_iso15924 (1000));
   g_assert_cmphex (0x41726162, ==, g_unicode_script_to_iso15924 (G_UNICODE_SCRIPT_ARABIC));
 
   g_assert_cmphex (G_UNICODE_SCRIPT_INVALID_CODE, ==, g_unicode_script_from_iso15924 (0));
   g_assert_cmphex (G_UNICODE_SCRIPT_UNKNOWN, ==, g_unicode_script_from_iso15924 (0x12345678));
-  g_assert_cmphex (G_UNICODE_SCRIPT_ARABIC, ==, g_unicode_script_from_iso15924 (0x41726162));
+
+#define PACK(a,b,c,d) ((guint32)((((guint8)(a))<<24)|(((guint8)(b))<<16)|(((guint8)(c))<<8)|((guint8)(d))))
+
+  for (i = 0; i < G_N_ELEMENTS (data); i++)
+    {
+      guint32 code = PACK (data[i].four_letter_code[0],
+                           data[i].four_letter_code[1],
+                           data[i].four_letter_code[2],
+                           data[i].four_letter_code[3]);
+
+      g_assert_cmphex (g_unicode_script_to_iso15924 (data[i].script), ==, code);
+      g_assert_cmpint (g_unicode_script_from_iso15924 (code), ==, data[i].script);
+    }
+
+#undef PACK
 }
 
 int
@@ -647,6 +844,8 @@ main (int   argc,
   g_test_add_func ("/unicode/mirror", test_mirror);
   g_test_add_func ("/unicode/mark", test_mark);
   g_test_add_func ("/unicode/title", test_title);
+  g_test_add_func ("/unicode/zero-width", test_zerowidth);
+  g_test_add_func ("/unicode/defined", test_defined);
   g_test_add_func ("/unicode/wide", test_wide);
   g_test_add_func ("/unicode/compose", test_compose);
   g_test_add_func ("/unicode/decompose", test_decompose);
@@ -655,6 +854,7 @@ main (int   argc,
   g_test_add_func ("/unicode/decompose-tail", test_decompose_tail);
   g_test_add_func ("/unicode/fully-decompose-len", test_fully_decompose_len);
   g_test_add_func ("/unicode/iso15924", test_iso15924);
+  g_test_add_func ("/unicode/cases", test_cases);
 
   return g_test_run();
 }
