@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #undef	G_LOG_DOMAIN
@@ -42,7 +44,6 @@ static gboolean toggle_ref3_strengthened;
 /*
  * TestObject, a parent class for TestObject
  */
-static GType test_object_get_type (void);
 #define TEST_TYPE_OBJECT          (test_object_get_type ())
 typedef struct _TestObject        TestObject;
 typedef struct _TestObjectClass   TestObjectClass;
@@ -56,7 +57,7 @@ struct _TestObjectClass
   GObjectClass parent_class;
 };
 
-G_DEFINE_TYPE (TestObject, test_object, G_TYPE_OBJECT)
+G_DEFINE_TYPE (TestObject, test_object, G_TYPE_OBJECT);
 
 static void
 test_object_finalize (GObject *object)
@@ -256,8 +257,7 @@ main (int   argc,
   g_assert (object_destroyed == FALSE);
 
   clear_flags ();
-  /* Check that removing a toggle ref with %NULL data works fine. */
-  g_object_remove_toggle_ref (object, toggle_ref2, NULL);
+  g_object_remove_toggle_ref (object, toggle_ref2, GUINT_TO_POINTER (24));
   g_assert (toggle_ref1_weakened == FALSE);
   g_assert (toggle_ref1_strengthened == FALSE);
   g_assert (toggle_ref2_weakened == FALSE);
